#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Drawing;

namespace HeuristicLab.Optimization.Networks {
  [Item("ServiceParameter", "A parameter of a service port.")]
  [StorableClass]
  public class ServiceParameter<T> : Entity, IServiceParameter<T> where T : class, IItem {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Field; }
    }
    public override Image ItemImage {
      get {
        if (Type == ServiceParameterType.Input) return HeuristicLab.Common.Resources.VSImageLibrary.ArrowDown;
        else if (Type == ServiceParameterType.Output) return HeuristicLab.Common.Resources.VSImageLibrary.ArrowUp;
        else return base.ItemImage;
      }
    }

    new public IPort Parent {
      get { return (IPort)base.Parent; }
      set {
        // NOTE: never call setter directly as the Parent property is set by the node which contains the port
        base.Parent = value;
      }
    }
    public Type DataType {
      get { return typeof(T); }
    }
    [Storable]
    private ServiceParameterType type;
    public ServiceParameterType Type {
      get { return type; }
      set {
        if (value != type) {
          type = value;
          OnTypeChanged();
        }
      }
    }
    [Storable]
    protected T value;
    public T Value {
      get { return value; }
      set {
        if (value != this.value) {
          DeregisterValueEvents();
          this.value = value;
          RegisterValueEvents();
          OnValueChanged();
        }
      }
    }
    IItem IServiceParameter.Value {
      get { return Value; }
      set {
        T val = value as T;
        if ((value != null) && (val == null))
          throw new InvalidOperationException(
            string.Format("Type mismatch. Value is not a \"{0}\".",
                          typeof(T).GetPrettyName())
          );
        Value = val;
      }
    }

    [StorableConstructor]
    protected ServiceParameter(bool deserializing) : base(deserializing) { }
    protected ServiceParameter(ServiceParameter<T> original, Cloner cloner)
      : base(original, cloner) {
      this.type = original.type;
      this.value = cloner.Clone(original.value);
      RegisterValueEvents();
    }
    public ServiceParameter() : base("ServiceParameter") { }
    public ServiceParameter(string name) : base(name) { }
    public ServiceParameter(string name, string description) : base(name, description) { }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterValueEvents();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ServiceParameter<T>(this, cloner);
    }

    public override string ToString() {
      var s = base.ToString();
      if (Value != null)
        s += ": " + Value.ToString();
      return s;
    }

    public event EventHandler TypeChanged;
    protected virtual void OnTypeChanged() {
      var handler = TypeChanged;
      if (handler != null) handler(this, EventArgs.Empty);
      OnItemImageChanged();
    }

    public event EventHandler ValueChanged;
    protected virtual void OnValueChanged() {
      var handler = ValueChanged;
      if (handler != null) handler(this, EventArgs.Empty);
      OnToStringChanged();
    }
    protected virtual void RegisterValueEvents() {
      if (value != null) {
        value.ToStringChanged += new EventHandler(Value_ToStringChanged);
      }
    }
    protected virtual void DeregisterValueEvents() {
      if (value != null) {
        value.ToStringChanged -= new EventHandler(Value_ToStringChanged);
      }
    }
    private void Value_ToStringChanged(object sender, EventArgs e) {
      OnToStringChanged();
    }
  }
}
